@@ -1,14 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,7 +46,7 @@ int get_doc_version(xmlDocPtr ldoc){
     goto fail;
   }
 
-  obj = xmlXPathEvalExpression((xmlChar *)"//cluster/@config_version", ctx);
+  obj = xmlXPathEvalExpression((xmlChar *)"/cluster/@config_version", ctx);
   if(!obj || !obj->nodesetval || (obj->nodesetval->nodeNr != 1)){
     log_err("Error while retrieving config_version.\n");
     error = -ENODATA;
@@ -123,7 +112,7 @@ char *get_cluster_name(xmlDocPtr ldoc){
     goto fail;
   }
 
-  obj = xmlXPathEvalExpression((xmlChar *)"//cluster/@name", ctx);
+  obj = xmlXPathEvalExpression((xmlChar *)"/cluster/@name", ctx);
   if(!obj || !obj->nodesetval || (obj->nodesetval->nodeNr != 1)){
     log_err("Error while retrieving config_version.\n");
     error = -ENODATA;
@@ -156,6 +145,3 @@ fail:
   EXIT("get_cluster_name");
   return rtn;
 }
-
-
-
