@@ -1,14 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2005-2008 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -564,9 +553,9 @@ static int get_ccs_join_info(void)
 	/* our nodename */
 	if (nodename_env) {
 		int ret;
-		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename);
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename_env);
 		if (ret < 0 || (size_t) ret >= sizeof(path)) {
-			log_printf(LOG_ERR, "Overridden node name %s is too long", nodename);
+			log_printf(LOG_ERR, "Overridden node name %s is too long", nodename_env);
 			write_cman_pipe("Overridden node name is too long");
 			error = -E2BIG;
 			goto out;
@@ -576,11 +565,12 @@ static int get_ccs_join_info(void)
 		if (!error) {
 			free(str);
 		} else {
-			log_printf(LOG_ERR, "Overridden node name %s is not in CCS", nodename);
+			log_printf(LOG_ERR, "Overridden node name %s is not in CCS", nodename_env);
 			write_cman_pipe("Overridden node name is not in CCS");
 			error = -ENOENT;
 			goto out;
 		}
+		strcpy(nodename, nodename_env);
 	} else {
 		struct utsname utsname;
 
