@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 /*
 * Implements Extendible Hashing as described in:
 *   "Extendible Hashing" by Fagin, et al in
@@ -59,12 +46,12 @@
 
 #include <linux/sched.h>
 #include <linux/slab.h>
-#include <linux/vmalloc.h>
 #include <linux/smp_lock.h>
 #include <linux/spinlock.h>
 #include <asm/semaphore.h>
 #include <linux/completion.h>
 #include <linux/buffer_head.h>
+#include <linux/vmalloc.h>
 
 #include "gfs.h"
 #include "dio.h"
@@ -76,9 +63,6 @@
 #include "rgrp.h"
 #include "trans.h"
 
-#define IS_LEAF     (1) /* Hashed (leaf) directory */
-#define IS_DINODE   (2) /* Linear (stuffed dinode block) directory */
-
 #if 1
 #define gfs_dir_hash2offset(h) (((uint64_t)(h)) >> 1)
 #define gfs_dir_offset2hash(p) ((uint32_t)(((uint64_t)(p)) << 1))
