@@ -1,15 +1,3 @@
-/*****************************************************************************
-******************************************************************************
-**
-**  Copyright (C) 2005-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-******************************************************************************
-*****************************************************************************/
-
 #include <unistd.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -80,7 +68,7 @@ void usage(char *name)
 void version(void)
 {
 	printf("GFS2 fsck %s (built %s %s)\n",
-	       GFS_RELEASE_NAME, __DATE__, __TIME__);
+	       RELEASE_VERSION, __DATE__, __TIME__);
 	printf("%s\n", REDHAT_COPYRIGHT);
 }
 
@@ -139,56 +127,26 @@ int read_cmdline(int argc, char **argv, struct gfs2_options *opts)
 
 void interrupt(int sig)
 {
-	fd_set rfds;
-	struct timeval tv;
 	char response;
-	int err;
+	char progress[PATH_MAX];
 
-	if (opts.query) /* if we're asking them a question */
-		return;     /* ignore the interrupt signal */
-	FD_ZERO(&rfds);
-	FD_SET(STDIN_FILENO, &rfds);
-
-	tv.tv_sec = 0;
-	tv.tv_usec = 0;
-	/* Make sure there isn't extraneous input before asking the
-	 * user the question */
-	while((err = select(STDIN_FILENO + 1, &rfds, NULL, NULL, &tv))) {
-		if(err < 0) {
-			log_debug("Error in select() on stdin\n");
-			break;
-		}
-		read(STDIN_FILENO, &response, sizeof(char));
+	if (!last_reported_block || last_reported_block == last_fs_block)
+		sprintf(progress, "progress unknown.\n");
+	else
+		sprintf(progress, "processing block %" PRIu64 " out of %"
+			PRIu64 "\n", last_reported_block, last_fs_block);
+	
+	response = generic_interrupt("gfs2_fsck", pass, progress,
+				     "Do you want to abort gfs_fsck, skip " \
+				     "the rest of this pass or continue " \
+				     "(a/s/c)?", "asc");
+	if(tolower(response) == 's') {
+		skip_this_pass = TRUE;
+		return;
 	}
-	while (TRUE) {
-		printf("\ngfs_fsck interrupted in %s:  ", pass);
-		if (!last_reported_block || last_reported_block == last_fs_block)
-			printf("progress unknown.\n");
-		else
-			printf("processing block %" PRIu64 " out of %" PRIu64 "\n",
-				   last_reported_block, last_fs_block);
-		printf("Do you want to abort gfs_fsck, skip the rest of %s or continue (a/s/c)?", pass);
-
-		/* Make sure query is printed out */
-		fflush(stdout);
-		read(STDIN_FILENO, &response, sizeof(char));
-
-		if(tolower(response) == 's') {
-			skip_this_pass = TRUE;
-			return;
-		}
-		else if (tolower(response) == 'a') {
-			fsck_abort = TRUE;
-			return;
-		}
-		else if (tolower(response) == 'c')
-			return;
-        else {
-			while(response != '\n')
-				read(STDIN_FILENO, &response, sizeof(char));
-			printf("Bad response, please type 'c', 'a' or 's'.\n");
-			continue;
-        }
+	else if (tolower(response) == 'a') {
+		fsck_abort = TRUE;
+		return;
 	}
 }
 
@@ -313,6 +271,7 @@ int main(int argc, char **argv)
 	struct gfs2_sbd sb;
 	struct gfs2_sbd *sbp = &sb;
 	int j;
+	enum update_flags update_sys_files;
 
 	memset(sbp, 0, sizeof(*sbp));
 
@@ -417,22 +376,24 @@ int main(int argc, char **argv)
 		else
 			log_notice("Pass5 complete      \n");
 	}
+	update_sys_files = (opts.no ? not_updated : updated);
 	/* Free up our system inodes */
-	inode_put(sbp->md.inum, updated);
-	inode_put(sbp->md.statfs, updated);
+	inode_put(sbp->md.inum, update_sys_files);
+	inode_put(sbp->md.statfs, update_sys_files);
 	for (j = 0; j < sbp->md.journals; j++)
-		inode_put(sbp->md.journal[j], updated);
-	inode_put(sbp->md.jiinode, updated);
-	inode_put(sbp->md.riinode, updated);
-	inode_put(sbp->md.qinode, updated);
-	inode_put(sbp->md.pinode, updated);
-	inode_put(sbp->md.rooti, updated);
-	inode_put(sbp->master_dir, updated);
+		inode_put(sbp->md.journal[j], update_sys_files);
+	inode_put(sbp->md.jiinode, update_sys_files);
+	inode_put(sbp->md.riinode, update_sys_files);
+	inode_put(sbp->md.qinode, update_sys_files);
+	inode_put(sbp->md.pinode, update_sys_files);
+	inode_put(sbp->md.rooti, update_sys_files);
+	inode_put(sbp->master_dir, update_sys_files);
 	if (lf_dip)
-		inode_put(lf_dip, updated);
+		inode_put(lf_dip, update_sys_files);
 /*	print_map(sbp->bl, sbp->last_fs_block); */
 
-	log_notice("Writing changes to disk\n");
+	if (!opts.no)
+		log_notice("Writing changes to disk\n");
 	bsync(sbp);
 	destroy(sbp);
 	log_notice("gfs2_fsck complete    \n");
