@@ -1,15 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <inttypes.h>
 #include <linux_endian.h>
 #include <stdlib.h>
@@ -52,7 +40,7 @@ int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 	if(gfs2_check_range(sbp, block)) {
 		log_err("Extended attributes indirect block out of range...removing\n");
 		ip->i_di.di_eattr = 0;
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
 	else if (gfs2_block_check(bl, block, &q)) {
@@ -62,7 +50,7 @@ int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
 	else if(q.block_type != gfs2_indir_blk) {
 		log_err("Extended attributes indirect block invalid...removing\n");
 		ip->i_di.di_eattr = 0;
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
 	else
@@ -77,12 +65,11 @@ int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 	int *update = (int *) private;
 	struct gfs2_sbd *sbp = ip->i_sbd;
 	struct gfs2_block_query q;
-	struct gfs2_buffer_head *leaf_bh;
 
 	if(gfs2_check_range(sbp, block)) {
 		log_err("Extended attributes block out of range...removing\n");
 		ip->i_di.di_eattr = 0;
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
 	else if (gfs2_block_check(bl, block, &q)) {
@@ -92,13 +79,12 @@ int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
 	else if(q.block_type != gfs2_meta_eattr) {
 		log_err("Extended attributes block invalid...removing\n");
 		ip->i_di.di_eattr = 0;
-		*update = 1;
+		*update = (opts.no ? not_updated : updated);
 		return 1;
 	}
 	else 
-		leaf_bh = bread(sbp, block);
+		*bh = bread(sbp, block);
 
-	*bh = leaf_bh;
 	return 0;
 }
 
@@ -237,26 +223,27 @@ int pass1c(struct gfs2_sbd *sbp)
 
 		if (skip_this_pass || fsck_abort) /* if asked to skip the rest */
 			return 0;
-		log_info("EA in inode %"PRIu64" (0x%" PRIx64 ")\n", block_no,
-				 block_no);
 		bh = bread(sbp, block_no);
-		ip = inode_get(sbp, bh);
+		if (gfs2_check_meta(bh, GFS2_METATYPE_IN)) { /* if a dinode */
+			log_info("EA in inode %"PRIu64" (0x%" PRIx64 ")\n",
+				 block_no, block_no);
+			gfs2_block_clear(bl, block_no, gfs2_eattr_block);
+			ip = fsck_inode_get(sbp, bh);
 
-		log_debug("Found eattr at %"PRIu64" (0x%" PRIx64 ")\n",
+			log_debug("Found eattr at %"PRIu64" (0x%" PRIx64 ")\n",
 				  ip->i_di.di_eattr, ip->i_di.di_eattr);
-		/* FIXME: Handle walking the eattr here */
-		error = check_inode_eattr(ip, &pass1c_fxns);
-		if(error < 0) {
-			stack;
-			return -1;
+			/* FIXME: Handle walking the eattr here */
+			error = check_inode_eattr(ip, &pass1c_fxns);
+			if(error < 0) {
+				stack;
+				brelse(bh, not_updated);
+				return -1;
+			}
+
+			fsck_inode_put(ip, update); /* dinode_out, brelse, free */
+		} else {
+			brelse(bh, update);
 		}
-
-		if(update)
-			gfs2_dinode_out(&ip->i_di, bh->b_data);
-
-		free(ip);
-		brelse(bh, update);
-
 		block_no++;
 	}
 	return 0;
