@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2006
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #define _MESSAGE_BUILD
 #include <message.h>
 #include <stdio.h>
@@ -79,10 +61,10 @@ do { \
 static int
 cluster_msg_send(msgctx_t *ctx, void *msg, size_t len)
 {
-	char buf[4096];
+	char ALIGNED buf[4096];
 	cluster_msg_hdr_t *h = (void *)buf;
-	int ret;
 	char *msgptr = (buf + sizeof(*h));
+	int ret;
 
 	errno = EINVAL;
 	if (ctx->type != MSG_CLUSTER)
@@ -235,9 +217,9 @@ cluster_send_control_msg(msgctx_t *ctx, int type)
 
 	swab_cluster_msg_hdr_t(&cm);
 
-	ret = (cman_send_data_unlocked((void *)&cm, sizeof(cm),
-			       ctx->u.cluster_info.nodeid,
-			       ctx->u.cluster_info.port, 0));
+	ret = (cman_send_data_unlocked((void *)&cm, sizeof(cm), 0,
+			       ctx->u.cluster_info.port,
+			       ctx->u.cluster_info.nodeid) );
 	return ret;
 }
 
@@ -249,6 +231,7 @@ static int
 cluster_msg_wait(msgctx_t *ctx, int timeout)
 {
 	struct timespec ts = {0, 0};
+	struct timeval tv = {0, 0};
 	int req = M_NONE;
 	int e;
 
@@ -261,9 +244,9 @@ cluster_msg_wait(msgctx_t *ctx, int timeout)
 		return -1;
 
 	if (timeout > 0) {
-		gettimeofday((struct timeval *)&ts, NULL);
-		ts.tv_sec += timeout;
-		ts.tv_nsec *= 1000;
+		gettimeofday(&tv, NULL);
+		ts.tv_sec = tv.tv_sec + timeout;
+		ts.tv_nsec = tv.tv_usec * 1000;
 	}
 
 	pthread_mutex_lock(&ctx->u.cluster_info.mutex);
@@ -589,6 +572,9 @@ cluster_msg_open(int type, int nodeid, int port, msgctx_t *ctx, int timeout)
 	ctx->u.cluster_info.remote_ctx = 0;
 	ctx->u.cluster_info.queue = NULL;
 
+	pthread_mutex_init(&ctx->u.cluster_info.mutex, NULL);
+	pthread_cond_init(&ctx->u.cluster_info.cond, NULL);
+
 	/* Assign context index */
 	if (assign_ctx(ctx) < 0) {
 		errno = EAGAIN;
@@ -607,7 +593,8 @@ cluster_msg_open(int type, int nodeid, int port, msgctx_t *ctx, int timeout)
 	/* Send open */
 	//printf("  Sending control message M_OPEN\n");
 	if (cluster_send_control_msg(ctx, M_OPEN) < 0) {
-		printf("Error sending control message\n");
+		printf("Error sending control message: %s\n", strerror(errno));
+		cluster_msg_close(ctx);
 		return -1;
 	}
 
@@ -1057,7 +1044,7 @@ cluster_msg_listen(int me, void *portp, msgctx_t **cluster_ctx)
 	errno = EINVAL;
 	if (!portp)
 		return -1;
-	port = *(int *)portp;
+	port = *(uint8_t *)portp;
 	if (port < 10 || port > 254)
 		return -1;
 
